function nn_result = MTS_Search(UTS_D,Index_D,window_size,queryname,thresholds, Ref_D,num_ref)
% MTS Search disk-aware version (Data and Index are both on the disk)

% PARAMETERS:
% Data is stored in UTS_D (UTS of ith variable are stored in the folder 'UTS_D\i')
% Index is stored in Index_D (Index of ith variable based on jth reference point is stored in the folder 'Index_D\i\j')
% window_size = the length of the sliding window, it should be smaller than the query length. 
% queryname = .mat file contain the variable query (a pre-generated query, details see the "Read in the query" section below)
% thresholds is an array, containining thresholds for each variable in the query
% Ref_D = directory contains reference point files for each variable (file 'RefPoints_Fi.txt' for ith variable)
% num_ref = number of ref points will be used

% This version uses reference points to find candidate sets for 1st and 2nd variables separately(first level filtering)
% then join these two candidate sets (second level filtering) and remove false alarms
% for the remaning variables, just simply do exact search based on the current candidate set. 

% Qiang Zhu, Summer 2009@NASA AMES 



%% Read in the query
query = load(queryname);
variables = query.variables; %variables is an array, containing dimensions of variables we want to query
num_var = length(variables);
q = query.q; %q{i} is a query subsequence on ith query variable
p = ones(size(variables,2),1);
%p = query.p; %lenghth of ith query varibale = p(i)* window_size
time_shiftings = query.time_shiftings; %time_shiftings is an array, containing time-shifting between variables

t_start = cputime;
%% Find candidate sets for 1st and 2nd variable
cand1 = FindCandidates(Index_D, q{1},variables(1),window_size, num_ref, thresholds(1),Ref_D);
num_cand = size(cand1,1);
disp('Number of candidates for 1st var:');disp(num_cand);

cand2 = FindCandidates(Index_D, q{2},variables(2),window_size, num_ref, thresholds(2),Ref_D);
num_cand = size(cand2,1);
disp('Number of candidates for 2nd var:');disp(num_cand);

%% Join two candidate sets from variable1 and variable2, considering the
%% time shifting
cand1and2 = join_cand(cand1, cand2, time_shiftings(1));
num_cand1and2 = size(cand1and2,1);
disp('Number of candidates for two variables after the join:');disp(num_cand1and2);

%% Performed an exact calculation for candidates in cand1and2
nn = cell(num_var,1);%nn{i} record results after checking i variables, so it's a n*(i+1) matirx(each row: mts followed by i offsets)
nn_num = zeros(num_var,1); %number of nn after checking i variables
nn{2} = zeros(size(cand1and2,1),3);

current_mts = 0;
i = 1;
while i <= size(cand1and2,1)
    if (mod(i,100)==0)
        fprintf('Processed %d\n',i);
    end
    mts = cand1and2(i,1);
    offset1 = cand1and2(i,2);
    offset2 = cand1and2(i,3);
    if mts ~= current_mts %if we haven't opened the file we need, first get its fid
        if current_mts ~= 0 %previous files are open
            fclose(fid1); fclose(fid2);
        end
        filename = [UTS_D '/' num2str(variables(1)) '/' num2str(mts) '.txt'];
        fid1 = fopen(filename,'rt');
        filename = [UTS_D '/' num2str(variables(2)) '/' num2str(mts) '.txt'];
        fid2 = fopen(filename,'rt');
        current_mts = mts;
    end
    %then count how many consecutive candidates in current spot
    n = 1; %number of consecutive candidates
    while i+1 <= size(cand1and2,1)&& cand1and2(i+1,1) == cand1and2(i,1) && cand1and2(i+1,2) == cand1and2(i,2) + 1 %if the next candidate is consecutive to current one
        n = n + 1;
        i = i + 1;
    end
    %read in the subsequence containing all these consecutive candidates
    for varf=1:offset1-1
        fgetl(fid1);
    end
    %fseek(fid1,(offset1-1)*(18),'bof');
    uts1 = fscanf(fid1,'%f',p(1)*window_size + n - 1);
    frewind(fid1);
    
    for varf=1:offset2-1
    	fgetl(fid2);
    end
    %fseek(fid2,(offset2-1)*(18),'bof');
    uts2 = fscanf(fid2,'%f',p(2)*window_size + n - 1);
    frewind(fid2);

    %do the exact calculation for these candidates
    for j = 1:n
%         Lp_dist(q{1}, uts1(j:j+p(1)*window_size-1),2) - thresholds(1)
        %Lp_dist(q{2}, uts2(j:j+p(2)*window_size-1),2) - thresholds(2)
        try
            if Lp_dist(q{1}, uts1(j:j+p(1)*window_size-1),2) <= thresholds(1)...
                    && Lp_dist(q{2}, uts2(j:j+p(2)*window_size-1),2) <= thresholds(2)
                nn_num(2) = nn_num(2) + 1;
                nn{2}(nn_num(2),:) = [mts, offset1+j-1, offset2+j-1];                
            end
        end
    end
    i = i + 1;
end
disp('Number of candidates after exact calculation:');disp(nn_num(2));

%% Then check remaining variables based on nn{2}
for var_i = 3:num_var
    nn{var_i} = zeros(nn_num(var_i-1),var_i+1);
    current_mts = 0;
    i = 1;
    while i <= nn_num(var_i-1)
        mts = nn{var_i-1}(i,1);
        offset = nn{var_i-1}(i,var_i)+ time_shiftings(var_i-1);
        if mts ~= current_mts
            if current_mts ~= 0 %a previous file is open
                fclose(fid);
            end
            filename = [UTS_D '/' num2str(variables(var_i)) '/' num2str(mts) '.txt'];
            fid = fopen(filename,'rt');
            current_mts = mts;
        end
        
        n = 1; %number of consecutive candidates
        while i+1 <= nn_num(var_i-1) && nn{var_i-1}(i+1,1) == nn{var_i-1}(i,1) && nn{var_i-1}(i+1,var_i) == nn{var_i-1}(i,var_i) + 1 %if the next candidate is consecutive to current one
            n = n + 1;
            i = i + 1;
        end
        %read in the subsequence containing all these consecutive candidates
        %fseek(fid,(offset-1)*(18),'bof');
        for varf=1:offset-1
           fgetl(fid);
        end
        uts = fscanf(fid,'%f',p(var_i)*window_size + n - 1);
        frewind(fid);
        %do the exact calculation for these candidates
        for j = 1:n
            try
                if Lp_dist(q{var_i}, uts(j:j+p(var_i)*window_size-1),2) <= thresholds(var_i)
                    nn_num(var_i) = nn_num(var_i)+1;
                    nn{var_i}(nn_num(var_i),:) = [nn{var_i-1}(i-(n-j),:), offset+j-1];
                end
            end
        end
        i = i + 1;
    end
end

disp('Total query time =');disp(cputime - t_start);
num_result = nn_num(num_var);
nn_result = nn{num_var}(1:num_result,:);
%save('nn_result.mat','nn_result');
disp('No of nn =');disp(size(nn_result,1));

